package com.ruoyi.business.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.business.domain.ResourceLibrary;
import com.ruoyi.business.service.IResourceLibraryService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 资源库管理Controller
 *
 * @author LiuChengRan
 * @date 2024-06-17
 */
@Api(tags = "资源库管理接口")
@RestController
@RequestMapping("/business/ctResourceLibrary")
public class ResourceLibraryController extends BaseController
{
    @Autowired
    private IResourceLibraryService resourceLibraryService;

    /**
     * 查询资源库管理列表
     */
    @ApiOperation(value="查询资源库管理列表", httpMethod = "GET", response = ResourceLibrary.class)
    @GetMapping("/list")
    public TableDataInfo list(ResourceLibrary resourceLibrary)
    {
        startPage();
        List<ResourceLibrary> list = resourceLibraryService.list(resourceLibrary);
        return getDataTable(list);
    }

    /**
     * 导出资源库管理列表
     */
    @ApiOperation(value="导出资源库管理列表", httpMethod = "POST")
    @Log(title = "资源库管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ResourceLibrary resourceLibrary)
    {
        List<ResourceLibrary> list = resourceLibraryService.list(resourceLibrary);
        ExcelUtil<ResourceLibrary> util = new ExcelUtil<ResourceLibrary>(ResourceLibrary.class);
        util.exportExcel(response, list, "资源库管理数据");
    }

    /**
     * 获取资源库管理详细信息
     */
    @ApiOperation(value="获取资源库管理详细信息", httpMethod = "GET", response = ResourceLibrary.class)
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(resourceLibraryService.getById(id));
    }

    /**
     * 新增资源库管理
     */
    @ApiOperation(value="新增资源库管理", httpMethod = "POST")
    @Log(title = "资源库管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody ResourceLibrary resourceLibrary)
    {
        return toAjax(resourceLibraryService.save(resourceLibrary));
    }

    /**
     * 修改资源库管理
     */
    @ApiOperation(value="修改资源库管理", httpMethod = "PUT")
    @Log(title = "资源库管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody ResourceLibrary resourceLibrary)
    {
        return toAjax(resourceLibraryService.updateById(resourceLibrary));
    }

    /**
     * 删除资源库管理
     */
    @ApiOperation(value="删除资源库管理", httpMethod = "DELETE")
    @Log(title = "资源库管理", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(resourceLibraryService.removeByIds(Arrays.asList(ids)));
    }
}
