package com.ruoyi.business.service.impl;

import java.text.ParseException;
import java.time.ZonedDateTime;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.business.domain.Apply;
import com.ruoyi.business.domain.Rebot;
import com.ruoyi.business.domain.ResourceLibrary;
import com.ruoyi.business.domain.bo.*;
import com.ruoyi.business.domain.vo.JobQueryLogVO;
import com.ruoyi.business.domain.vo.JobQueryVO;
import com.ruoyi.business.domain.vo.JobStartVO;
import com.ruoyi.business.domain.vo.ListRebotVO;
import com.ruoyi.business.service.IApplyService;
import com.ruoyi.business.service.IRebotService;
import com.ruoyi.business.service.IResourceLibraryService;
import com.ruoyi.business.util.YinDaoHttpUtils;
import com.ruoyi.business.yddoman.BaseDTO;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.enums.ExcTypeStatus;
import com.ruoyi.common.enums.PlanRunStatus;
import com.ruoyi.common.enums.RebotStatus;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.support.CronSequenceGenerator;
import org.springframework.stereotype.Service;
import com.ruoyi.business.mapper.ApplyPlanMapper;
import com.ruoyi.business.domain.ApplyPlan;
import com.ruoyi.business.service.IApplyPlanService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;

/**
 * 应用执行计划管理Service业务层处理
 *
 * @author LiuChengRan
 * @date 2024-06-14
 */
@Service
public class ApplyPlanServiceImpl extends ServiceImpl<ApplyPlanMapper, ApplyPlan> implements IApplyPlanService {
    @Autowired
    private IApplyService applyService;

    @Autowired
    private IRebotService rebotService;

    @Autowired
    private IResourceLibraryService resourceLibraryService;

    /**
     * 重新运行
     *
     * @param jobQueryBO
     * @return
     * @throws IllegalAccessException
     */
    @Override
    public boolean appRetry(JobQueryBO jobQueryBO) throws IllegalAccessException {
        BaseDTO baseDTO = YinDaoHttpUtils.appRetry(jobQueryBO);
        if (baseDTO.getSuccess()) {
            return baseMapper.appRetry(jobQueryBO.getJobUuid()) > 0;
        } else {
            return false;
        }
    }

    /**
     * 停止计划运行
     *
     * @return
     */
    @Override
    public boolean stopRun(JobQueryBO jobQueryBO) throws IllegalAccessException {
        BaseDTO baseDTO = YinDaoHttpUtils.appStop(jobQueryBO);
        return baseDTO.getSuccess();
    }

    /**
     * 指定时间执行
     *
     * @return
     */
    @Override
    @Transactional
    public boolean runAppBySetTime() throws IllegalAccessException {
        List<ApplyPlan> applyPlanList = this.lambdaQuery().eq(ApplyPlan::getExcType, ExcTypeStatus.TWO.getKey()).isNull(ApplyPlan::getTaskUuid).orderByAsc(ApplyPlan::getExcTime).list();
        if (applyPlanList.isEmpty()) {
            return false;
        }

        ListRebotBO listRebotBO = new ListRebotBO();
        listRebotBO.setStatus(RebotStatus.IDLE.getKey());
        listRebotBO.setPage(1);
        listRebotBO.setSize(100);
        // 只获取空闲的机器人
        List<ListRebotVO> listRebotVos = YinDaoHttpUtils.listRebot(listRebotBO);
        if (listRebotVos.isEmpty()) {
            log.debug("暂无空闲机器人");
            // 刷新机器人数据
//            rebotService.syn(new Rebot());
            return false;
        }
        listRebotVos.forEach(rebot -> {
            applyPlanList.forEach(e -> {
                if (e.getExcTime().getTime() <= System.currentTimeMillis() && PlanRunStatus.AWAIT_CREATE.getKey().equals(e.getTaskStatus())) {
                    ApplyStartBO applyStartBO = new ApplyStartBO();
                    applyStartBO.setRobotUuid(e.getAppId());
                    applyStartBO.setAccountName(rebot.getRobotClientName());
                    applyStartBO.setManualTime(e.getManualTime());
                    //high 高 middle 中 low 低
                    applyStartBO.setPriority("high");
                    // 如果应用支持参数
                    if (!Objects.isNull(e.getSupportParam()) && e.getSupportParam() == 1) {
                        if (StringUtils.isNotEmpty(e.getPlanParams())) {
                            applyStartBO.setPlanParams(e.getPlanParams());
                            List<ApplyStartBO.RobotParam> paramList = JSON.parseArray(e.getPlanParams(), ApplyStartBO.RobotParam.class);
                            applyStartBO.setPlanParamsList(paramList);
                        }
                    }
                    try {
                        JobStartVO jobStartVO = YinDaoHttpUtils.appStart(applyStartBO);
                        ApplyPlan applyPlan = new ApplyPlan();
                        applyPlan.setId(e.getId());
                        applyPlan.setUpdateBy("系统修改");
                        applyPlan.setUpdateTime(new Date());
                        applyPlan.setTaskUuid(jobStartVO.getJobUuid());
                        applyPlan.setTaskStatus(PlanRunStatus.CREATED.getKey());
                        this.updateById(applyPlan);
                        Rebot updateRebot = new Rebot();
                        updateRebot.setUpdateBy("系统修改");
                        updateRebot.setUpdateTime(new Date());
                        updateRebot.setStatus(RebotStatus.RUNNING.getKey());
                        if (rebotService.lambdaUpdate().eq(Rebot::getRobotClientName, rebot.getRobotClientName()).update(updateRebot)) {
                            applyPlanList.remove(e);
                        }
                    } catch (IllegalAccessException ex) {
                        throw new RuntimeException(ex);
                    }
                }
            });
        });

        return false;
    }

    /**
     * 表达式执行
     */
    @Override
    public boolean runAppByCron() throws IllegalAccessException {
        // 先查计划，避免每次命中第三方接口
        List<ApplyPlan> applyPlanList = this.lambdaQuery().eq(ApplyPlan::getExcType, ExcTypeStatus.TREE.getKey()).list();
        if (applyPlanList.isEmpty()) {
            return false;
        }
        // 需要执行的计划
        List<ApplyPlan> execApplyPlan = new ArrayList<>();
        applyPlanList.forEach(e -> {
            try {
                CronExpression cron = new CronExpression(e.getCronExpression());
                // 如果该计划没有执行过
                if (Objects.isNull(e.getLastExecTime())) {
                    // 判断当前时间是否满足表达式
                    Date nextDate = cron.getNextValidTimeAfter(new Date());
                    if (new Date().compareTo(nextDate) > -1) {
                        execApplyPlan.add(e);
                    }
                } else {
                    // 根据表达式返回下一个执行时间
                    Date nextDate = cron.getNextValidTimeAfter(e.getNextExecTime());
                    if (new Date().compareTo(nextDate) > -1) {
                        execApplyPlan.add(e);
                    }
                }
            } catch (ParseException ex) {
                log.error(e.getPlanName() + "的时间表达式错误！");
                throw new RuntimeException(ex);
            }
        });
        if (execApplyPlan.isEmpty()) {
            log.debug("没有待执行的计划");
            return false;
        }

        ListRebotBO listRebotBO = new ListRebotBO();
        listRebotBO.setStatus(RebotStatus.IDLE.getKey());
        listRebotBO.setPage(1);
        listRebotBO.setSize(100);
        // 只获取空闲的机器人
        List<ListRebotVO> listRebotVos = YinDaoHttpUtils.listRebot(listRebotBO);
        if (listRebotVos.isEmpty()) {
            log.debug("暂无空闲机器人");
            // 刷新机器人数据
//            rebotService.syn(new Rebot());
            return false;
        }
        listRebotVos.forEach(rebot -> {
            execApplyPlan.forEach(e -> {
                // 该计划满足执行条件
                ApplyStartBO applyStartBO = new ApplyStartBO();
                applyStartBO.setRobotUuid(e.getAppId());
                applyStartBO.setAccountName(rebot.getRobotClientName());
                applyStartBO.setManualTime(e.getManualTime());
                //high 高 middle 中 low 低
                applyStartBO.setPriority("high");

                // 如果应用支持参数
                if (!Objects.isNull(e.getSupportParam()) && e.getSupportParam() == 1) {
                    // 获取参数
                    if (StringUtils.isNotEmpty(e.getPlanParams())) {
                        applyStartBO.setPlanParams(e.getPlanParams());
                        List<ApplyStartBO.RobotParam> paramList = JSON.parseArray(e.getPlanParams(), ApplyStartBO.RobotParam.class);
                        applyStartBO.setPlanParamsList(paramList);
                    }
                }
                String planParams = e.getPlanParams();
                if (StringUtils.isNotEmpty(planParams)) {
                    JSONArray jsonArray = JSON.parseArray(planParams);
                    // 直接转换成List
                    List<ApplyStartBO.RobotParam> list = jsonArray.toJavaList(ApplyStartBO.RobotParam.class);
                    applyStartBO.setPlanParamsList(list);
                }
                try {
                    JobStartVO jobStartVO = YinDaoHttpUtils.appStart(applyStartBO);
                    ApplyPlan applyPlan = new ApplyPlan();
                    applyPlan.setId(e.getId());
                    applyPlan.setUpdateBy("系统修改");
                    applyPlan.setUpdateTime(new Date());
                    applyPlan.setTaskUuid(jobStartVO.getJobUuid());
                    applyPlan.setTaskStatus(PlanRunStatus.CREATED.getKey());
                    // 根据表达式返回下一个执行时间
                    CronExpression cron = new CronExpression(e.getCronExpression());
                    Date nextDate = null;
                    if (Objects.isNull(e.getLastExecTime())) {
                        nextDate = cron.getNextValidTimeAfter(new Date());
                    } else {
                        nextDate = cron.getNextValidTimeAfter(e.getLastExecTime());
                    }
                    Date date = new Date();
                    applyPlan.setLastExecTime(date);
                    applyPlan.setNextExecTime(nextDate);
                    this.updateById(applyPlan);
                    Rebot updateRebot = new Rebot();
                    updateRebot.setUpdateBy("系统修改");
                    updateRebot.setUpdateTime(new Date());
                    updateRebot.setStatus(RebotStatus.RUNNING.getKey());
                    if (rebotService.lambdaUpdate().eq(Rebot::getRobotClientName, rebot.getRobotClientName()).update(updateRebot)) {
                        applyPlanList.remove(e);
                    }
                } catch (IllegalAccessException | ParseException ex) {
                    throw new RuntimeException(ex);
                }
            });
        });

        return false;
    }


    /**
     * 获取应用执行结果
     *
     * @return
     * @throws IllegalAccessException
     */
    @Override
    @Transactional
    public void queryAppStartResult() throws IllegalAccessException {
        // 不查询的状态
        List<String> noStatus = new ArrayList<>();
        noStatus.add(PlanRunStatus.FINISH.getKey());
        noStatus.add(PlanRunStatus.STOPPED.getKey());
        noStatus.add(PlanRunStatus.ERROR.getKey());
        noStatus.add(PlanRunStatus.SKIPPED.getKey());
        noStatus.add(PlanRunStatus.CANCEL.getKey());
        // 查询任务运行状态非空非终态并且已经创建的任务
        List<ApplyPlan> list = this.lambdaQuery().notIn(ApplyPlan::getTaskStatus, noStatus).isNotNull(ApplyPlan::getTaskStatus).orderByAsc(ApplyPlan::getPriority).list();
        if (list.isEmpty()) {
            log.debug("没有需要更新状态的计划任务");
            return;
        }
        for (ApplyPlan applyPlan : list) {
            JobQueryBO jobQueryBO = new JobQueryBO();
            jobQueryBO.setJobUuid(applyPlan.getTaskUuid());
            JobQueryVO jobQueryVO = YinDaoHttpUtils.queryAppStartResult(jobQueryBO);
            // 如果状态不一致，更新最新状态
            if (jobQueryVO != null && !applyPlan.getTaskStatus().equals(jobQueryVO.getStatus())) {
                // 修改的任务计划
                ApplyPlan updateApplyPlan = new ApplyPlan();
                updateApplyPlan.setTaskStatus(jobQueryVO.getStatus());
                if (null != jobQueryVO.getEndTime()) {
                    updateApplyPlan.setEndTime(jobQueryVO.getEndTime());
                    updateApplyPlan.setStartTime(jobQueryVO.getStartTime());
                    updateApplyPlan.setEndTime(jobQueryVO.getEndTime());
                    long diff = updateApplyPlan.getEndTime().getTime() - updateApplyPlan.getStartTime().getTime();
                    long planTime = diff / (60 * 1000);
                    updateApplyPlan.setPlanTime(planTime + "");
                    if (!Objects.isNull(applyPlan.getManualTime())) {
                        updateApplyPlan.setTimeSaving((Long.parseLong(applyPlan.getManualTime()) - planTime) + "");
                    }
                }
                updateApplyPlan.setId(applyPlan.getId());
                updateApplyPlan.setUpdateBy("系统修改");
                updateApplyPlan.setUpdateTime(new Date());

                // 如果是终态并且有响应参数 获取返回值
                List<String> finishStatus = Arrays.asList(PlanRunStatus.FINISH.getKey(), PlanRunStatus.STOPPED.getKey(), PlanRunStatus.ERROR.getKey(), PlanRunStatus.SKIPPED.getKey(), PlanRunStatus.CANCEL.getKey());
                if (finishStatus.contains(jobQueryVO.getStatus())) {
                    List<JobQueryVO.RobotParam> outputs = jobQueryVO.getRobotParams().getOutputs();
                    if (null != outputs && !outputs.isEmpty()) {
                        JobQueryVO.RobotParam robotParam = outputs.get(0);
                        updateApplyPlan.setOutParam(robotParam.getValue());
                    }
                }
                updateApplyPlan.setRemark(jobQueryVO.getRemark());
                this.updateById(updateApplyPlan);
            }
            // 如果是终态改变机器人状态
            List<String> finishStatus = Arrays.asList(PlanRunStatus.FINISH.getKey(), PlanRunStatus.STOPPED.getKey(), PlanRunStatus.ERROR.getKey(), PlanRunStatus.SKIPPED.getKey(), PlanRunStatus.CANCEL.getKey());
            if (finishStatus.contains(jobQueryVO.getStatus())) {
                Rebot rebot = new Rebot();
                rebot.setStatus(RebotStatus.IDLE.getKey());
                rebot.setUpdateBy("系统修改");
                rebot.setUpdateTime(new Date());
                rebot.setStatus(RebotStatus.IDLE.getKey());
                rebotService.lambdaUpdate().eq(Rebot::getRobotClientName, jobQueryVO.getRobotClientName()).update(rebot);
            } else if (PlanRunStatus.RUNNING.getKey().equals(jobQueryVO.getStatus())) {
                Rebot rebot = new Rebot();
                rebot.setStatus(RebotStatus.RUNNING.getKey());
                rebot.setUpdateBy("系统修改");
                rebot.setUpdateTime(new Date());
                rebotService.lambdaUpdate().eq(Rebot::getRobotClientName, jobQueryVO.getRobotClientName()).update(rebot);
            }
        }
    }

    /**
     * 执行计划（针对立即执行的计划）
     *
     * @param applyStartBO 该对象只有 应用id 优先级和人工用时
     * @return
     */
    @Override
    @Transactional
    public boolean execPlan(ApplyStartBO applyStartBO) throws IllegalAccessException {
        // 先查计划 避免每次直接命中第三方接口
        Map<Long, List<ApplyPlan>> applyPlanMap = this.lambdaQuery().eq(ApplyPlan::getExcType, ExcTypeStatus.ONE.getKey()).isNull(ApplyPlan::getTaskUuid)
                .orderByAsc(ApplyPlan::getPriority).list().stream().collect(Collectors.groupingBy(ApplyPlan::getDeptId));

        // 查询所有待执行的任务
        if (null == applyStartBO && applyPlanMap.isEmpty()) {
            log.debug("没有等待执行的计划");
            return false;
        }

        ListRebotBO listRebotBO = new ListRebotBO();
        listRebotBO.setStatus(RebotStatus.IDLE.getKey());
        listRebotBO.setPage(1);
        listRebotBO.setSize(100);
        // 只获取空闲的机器人
        List<ListRebotVO> listRebotVos = YinDaoHttpUtils.listRebot(listRebotBO);
        if (listRebotVos.isEmpty()) {
            log.debug("暂无空闲机器人");
            // 刷新机器人数据
//            rebotService.syn(new Rebot());
            return false;
        }

        // 获取每个部门优先级最高的计划
        List<ApplyPlan> applyPlans = new ArrayList<>();
        applyPlanMap.forEach((k, v) -> {
            v.sort((Comparator.comparingInt(ApplyPlan::getPriority)));
            applyPlans.add(v.get(0));
        });
        if (null == applyStartBO && applyPlans.isEmpty()) {
            log.debug("没有等待执行的计划");
            return false;
        }
        applyPlans.sort((Comparator.comparingInt(ApplyPlan::getPriority)));

        int i = 0;
        for (ListRebotVO listRebotVO : listRebotVos) {
            ApplyPlan applyPlan = null;
            if (applyPlans.isEmpty()) {
                log.debug("没有等待执行的计划！");
                continue;
            }
            if (null == applyStartBO) {
                applyPlan = applyPlans.get(0);
                applyStartBO = new ApplyStartBO();
                applyStartBO.setRobotUuid(applyPlan.getAppId());
                //high 高 middle 中 low 低
                applyStartBO.setPriority("high");
            } else {
                applyPlan = new ApplyPlan();
                BeanUtils.copyBeanProp(applyPlan, applyStartBO);
                // 运行的应用上一步已给值，此处无需再给
            }
            applyStartBO.setAccountName(listRebotVO.getRobotClientName());
            // 如果应用支持参数
            if (!Objects.isNull(applyPlan.getSupportParam()) && applyPlan.getSupportParam() == 1) {
                if (StringUtils.isNotEmpty(applyPlan.getPlanParams())) {
                    applyStartBO.setPlanParams(applyPlan.getPlanParams());
                    List<ApplyStartBO.RobotParam> paramList = JSON.parseArray(applyPlan.getPlanParams(), ApplyStartBO.RobotParam.class);
                    applyStartBO.setPlanParamsList(paramList);
                }
            }
            JobStartVO jobStartVO = YinDaoHttpUtils.appStart(applyStartBO);
            ApplyPlan updateApplyPlan = new ApplyPlan();
            updateApplyPlan.setId(applyPlan.getId());
            updateApplyPlan.setCreateBy("系统创建");
            updateApplyPlan.setCreateTime(new Date());
            updateApplyPlan.setUpdateBy("系统修改");
            updateApplyPlan.setUpdateTime(new Date());
            updateApplyPlan.setStartTime(new Date());
            updateApplyPlan.setTaskUuid(jobStartVO.getJobUuid());
            updateApplyPlan.setPlanName(applyPlan.getPlanName());
            updateApplyPlan.setTaskStatus(PlanRunStatus.CREATED.getKey());
            updateApplyPlan.setRobotName(listRebotVO.getRobotClientName());
            updateApplyPlan.setManualTime(applyPlan.getManualTime());
            updateApplyPlan.setDeptId(applyStartBO.getDeptId());
            if (this.saveOrUpdate(updateApplyPlan)) {
                applyPlans.remove(0);
                i++;
            }
        }
        return i > 0;
    }


    @Override
    @Transactional
    public boolean save(AddApplyPlanBO addApplyPlanBO) {
        List<ApplyPlan> applyPlanList = new ArrayList<>();
        Map<String, List<Apply>> appMap = applyService.lambdaQuery().in(Apply::getAppId, addApplyPlanBO.getAppId()).list().stream().collect(Collectors.groupingBy(Apply::getAppId));
        // 是否有空闲机器人并且创建计划成功
        AtomicBoolean insert = new AtomicBoolean(false);

        addApplyPlanBO.getAppId().forEach(e -> {
            ApplyPlan applyPlan = new ApplyPlan();
            BeanUtils.copyBeanProp(applyPlan, addApplyPlanBO);
            Apply apply = appMap.get(e).get(0);
            applyPlan.setAppName(apply.getAppName());
            applyPlan.setAppId(apply.getAppId());
            applyPlan.setAppType(apply.getAppType());
            applyPlan.setAppTypeName(apply.getAppTypeName());
            applyPlan.setManualTime(apply.getManualTime());
            if (null != apply.getSupportParam() && apply.getSupportParam() == 1) {
                // 获取参数 每一次执行计划的参数都是独立的
                List<ApplyStartBO.RobotParam> paramList = resourceLibraryService.lambdaQuery().eq(ResourceLibrary::getAppId, apply.getAppId()).list().stream()
                        .map(item -> {
                            ApplyStartBO.RobotParam robotParam = new ApplyStartBO.RobotParam();
                            robotParam.setName(item.getResourceName());
                            robotParam.setValue(item.getResourceValue());
                            robotParam.setType(item.getResourceType());
                            return robotParam;
                        }).collect(Collectors.toList());
                if (!paramList.isEmpty()) {
                    // json集合转字符串
                    applyPlan.setPlanParams(JSONArray.toJSONString(paramList));
                }
            }
            applyPlan.setDeptId(SecurityUtils.getDeptId());

            try {
                // 只指定优先级和应用和人工用时 机器人等下一个阶段判读是否有空闲
                ApplyStartBO applyStartBO = new ApplyStartBO();
                applyStartBO.setRobotUuid(applyPlan.getAppId());
                //high 高 middle 中 low 低
                applyStartBO.setPriority("high");
                applyStartBO.setManualTime(applyPlan.getManualTime());
                applyStartBO.setPlanName(addApplyPlanBO.getPlanName());
                insert.set(execPlan(applyStartBO));
            } catch (Exception ex) {
                log.error("分配机器人异常：" + ex.getMessage());
            }
            if (!insert.get()) {
                applyPlan.setTaskStatus(PlanRunStatus.AWAIT_CREATE.getKey());
                applyPlanList.add(applyPlan);
            }
        });
        return this.saveBatch(applyPlanList) || insert.get();
    }

    @Override
    public boolean update(AddApplyPlanBO addApplyPlanBO) {
        List<ApplyPlan> applyPlanList = new ArrayList<>();
        Map<String, List<Apply>> appMap = applyService.lambdaQuery().in(Apply::getAppId, addApplyPlanBO.getAppId()).eq(Apply::getDeptId, SecurityUtils.getDeptId()).list().stream().collect(Collectors.groupingBy(Apply::getAppId));
        addApplyPlanBO.getAppId().forEach(e -> {
            ApplyPlan applyPlan = new ApplyPlan();
            BeanUtils.copyBeanProp(applyPlan, addApplyPlanBO);
            Apply apply = appMap.get(e).get(0);
            applyPlan.setAppName(apply.getAppName());
            applyPlan.setAppId(apply.getAppId());
            applyPlan.setAppType(apply.getAppType());
            applyPlan.setAppTypeName(apply.getAppTypeName());
            applyPlanList.add(applyPlan);
        });

        return super.updateBatchById(applyPlanList);
    }

    /**
     * 查询应用执行计划管理列表
     *
     * @param applyPlan 应用执行计划管理
     * @return 应用执行计划管理
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<ApplyPlan> list(ApplyPlan applyPlan) {
        return baseMapper.selectApplyPlanList(applyPlan);
    }
}
